@@ -31,6 +31,7 @@
 #include "arg.h"
 #include "repo.h"
 #include "function.h"
+#include "proxyutils.h"
 #include "param.h"
 #include "value.h"
 #include "keep-alive.h"
@@ -73,7 +74,7 @@ enum {
 
 static struct JSClass gjs_object_instance_class;
 
-GJS_DEFINE_DYNAMIC_PRIV_FROM_JS(ObjectInstance, gjs_object_instance_class)
+GJS_DEFINE_PRIV_FROM_JS(ObjectInstance, gjs_object_instance_class)
 
 static JSObject*       peek_js_obj  (JSContext *context,
                                      GObject   *gobj);
@@ -1441,10 +1442,6 @@ emit_func(JSContext *context,
     return ret;
 }
 
-/* Default spidermonkey toString is worthless.  Replace it
- * with something that gives us both the introspection name
- * and a memory address.
- */
 static JSBool
 to_string_func(JSContext *context,
                uintN      argc,
@@ -1452,45 +1449,31 @@ to_string_func(JSContext *context,
 {
     JSObject *obj = JS_THIS_OBJECT(context, vp);
     ObjectInstance *priv;
-    char *strval;
-    JSBool ret;
-    const char *namespace;
-    const char *name;
+    JSBool ret = JS_FALSE;
     jsval retval;
 
     if (!do_base_typecheck(context, obj, JS_TRUE))
-        return JS_FALSE;
+        goto out;
 
     priv = priv_from_js(context, obj);
 
     if (priv == NULL) {
         throw_priv_is_null_error(context);
-        return JS_FALSE; /* wrong class passed in */
+        goto out;  /* wrong class passed in */
     }
+    
+    if (!_gjs_proxy_to_string_func(context, obj, "object", (GIBaseInfo*)priv->info,
+                                   priv->gtype, priv->gobj, &retval))
+        goto out;
 
-    if (priv->info) {
-        namespace = g_base_info_get_namespace( (GIBaseInfo*) priv->info);
-        name = g_base_info_get_name( (GIBaseInfo*) priv->info);
-    } else {
-        namespace = "";
-        name = g_type_name(priv->gtype);
-    }
-
-    if (priv->gobj == NULL) {
-        strval = g_strdup_printf ("[object prototype of GIName:%s.%s jsobj@%p]", namespace, name, obj);
-    } else {
-        strval = g_strdup_printf ("[object instance proxy GIName:%s.%s jsobj@%p native@%p]", namespace, name, obj, priv->gobj);
-    }
-
-    ret = gjs_string_from_utf8 (context, strval, -1, &retval);
-    if (ret)
-        JS_SET_RVAL(context, vp, retval);
-    g_free (strval);
+    ret = JS_TRUE;
+    JS_SET_RVAL(context, vp, retval);
+ out:
     return ret;
 }
 
 static struct JSClass gjs_object_instance_class = {
-    NULL, /* We copy this class struct with multiple names */
+    "GObject_Object",
     JSCLASS_HAS_PRIVATE |
     JSCLASS_NEW_RESOLVE |
     JSCLASS_NEW_RESOLVE_GETS_START |
@@ -1718,34 +1701,23 @@ gjs_define_object_class(JSContext     *context,
         ns = g_base_info_get_namespace((GIBaseInfo*) info);
     else
         ns = "unknown";
-    prototype = gjs_init_class_dynamic(context, in_object,
-                                       /* parent prototype JSObject* for
-                                        * prototype; NULL for
-                                        * Object.prototype
-                                        */
-                                       parent_proto,
-                                       ns,
-                                       constructor_name,
-                                       &gjs_object_instance_class,
-                                       /* constructor for instances (NULL for
-                                        * none - just name the prototype like
-                                        * Math - rarely correct)
-                                        */
-                                       gjs_object_instance_constructor,
-                                       /* number of constructor args */
-                                       0,
-                                       /* props of prototype */
-                                       parent_proto ? NULL : &gjs_object_instance_proto_props[0],
-                                       /* funcs of prototype */
-                                       parent_proto ? NULL : &gjs_object_instance_proto_funcs[0],
-                                       /* props of constructor, MyConstructor.myprop */
-                                       NULL,
-                                       /* funcs of constructor, MyConstructor.myfunc() */
-                                       NULL);
-    if (prototype == NULL)
+    if (!gjs_init_class_dynamic(context, in_object,
+                                parent_proto,
+                                ns, constructor_name,
+                                &gjs_object_instance_class,
+                                gjs_object_instance_constructor, 0,
+                                /* props of prototype */
+                                parent_proto ? NULL : &gjs_object_instance_proto_props[0],
+                                /* funcs of prototype */
+                                parent_proto ? NULL : &gjs_object_instance_proto_funcs[0],
+                                /* props of constructor, MyConstructor.myprop */
+                                NULL,
+                                /* funcs of constructor, MyConstructor.myfunc() */
+                                NULL,
+                                &prototype,
+                                &constructor)) {
         gjs_fatal("Can't init class %s", constructor_name);
-
-    g_assert(gjs_object_has_property(context, in_object, constructor_name));
+    }
 
     GJS_INC_COUNTER(object);
     priv = g_slice_new0(ObjectInstance);
@@ -1759,25 +1731,8 @@ gjs_define_object_class(JSContext     *context,
     gjs_debug(GJS_DEBUG_GOBJECT, "Defined class %s prototype %p class %p in object %p",
               constructor_name, prototype, JS_GET_CLASS(context, prototype), in_object);
 
-    /* Now get the constructor we defined in
-     * gjs_init_class_dynamic
-     */
-    gjs_object_get_property(context, in_object, constructor_name, &value);
-    constructor = NULL;
-    if (!JSVAL_IS_VOID(value)) {
-       if (!JSVAL_IS_OBJECT(value)) {
-            gjs_throw(context, "Property '%s' does not look like a constructor",
-                      constructor_name);
-            if (info)
-                g_base_info_unref((GIBaseInfo*)info);
-            return FALSE;
-       }
-
-       constructor = JSVAL_TO_OBJECT(value);
-
-       if (info)
-           gjs_define_static_methods(context, constructor, gtype, info);
-    }
+    if (info)
+        gjs_define_static_methods(context, constructor, gtype, info);
 
     value = OBJECT_TO_JSVAL(gjs_gtype_create_gtype_wrapper(context, gtype));
     JS_DefineProperty(context, constructor, "$gtype", value,
