@@ -37,6 +37,8 @@
 #pragma GCC diagnostic pop
 #include <glib.h>
 
+#include <gjs/jsapi-util.h>
+
 G_BEGIN_DECLS
 
 /* This file inspects jsapi.h and attempts to provide a compatibility shim.
@@ -66,18 +68,17 @@ gjs_##name##_constructor(JSContext  *context,           \
  * GJS_NATIVE_CONSTRUCTOR_PRELUDE:
  * Call after the initial variable declaration.
  */
-#define GJS_NATIVE_CONSTRUCTOR_PRELUDE(name)                                         \
-    {                                                                                \
-        if (!JS_IsConstructing(context, vp)) {  \
-            gjs_throw_constructor_error(context);                                    \
-            return JS_FALSE;                                                         \
-        }                                                                            \
-        object = JS_NewObjectForConstructor(context, vp);                            \
-        if (object == NULL)                                                          \
-            return JS_FALSE;                                                         \
+#define GJS_NATIVE_CONSTRUCTOR_PRELUDE(name)                            \
+    {                                                                   \
+        if (!JS_IsConstructing(context, vp)) {                          \
+            gjs_throw_constructor_error(context);                       \
+            return JS_FALSE;                                            \
+        }                                                               \
+        object = gjs_new_object_for_constructor(context, &gjs_##name##_class, vp); \
+        if (object == NULL)                                             \
+            return JS_FALSE;                                            \
     }
 
-
 /**
  * GJS_NATIVE_CONSTRUCTOR_FINISH:
  * Call this at the end of a constructor when it's completed
@@ -86,6 +87,19 @@ gjs_##name##_constructor(JSContext  *context,           \
 #define GJS_NATIVE_CONSTRUCTOR_FINISH(name)             \
     JS_SET_RVAL(context, vp, OBJECT_TO_JSVAL(object));
 
+/**
+ * GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT:
+ * Defines a constructor whose only purpose is to throw an error
+ * and fail. To be used with classes that require a constructor (because they have
+ * instances), but whose constructor cannot be used from JS code.
+ */
+#define GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT(name)            \
+    GJS_NATIVE_CONSTRUCTOR_DECLARE(name)                        \
+    {                                                           \
+        gjs_throw_abstract_constructor_error(context, vp);      \
+        return JS_FALSE;                                        \
+    }
+
 G_END_DECLS
 
 #endif  /* __GJS_COMPAT_H__ */
