@@ -53,6 +53,7 @@ GJS_DEFINE_PRIV_FROM_JS(Importer, gjs_importer_class)
 static JSBool
 define_meta_properties(JSContext  *context,
                        JSObject   *module_obj,
+                       const char *full_path,
                        const char *module_name,
                        JSObject   *parent)
 {
@@ -66,6 +67,18 @@ define_meta_properties(JSContext  *context,
     gjs_debug(GJS_DEBUG_IMPORTER, "Defining parent %p of %p '%s' is mod %d",
               parent, module_obj, module_name ? module_name : "<root>", parent_is_module);
 
+    if (full_path != NULL) {
+        if (!JS_DefineProperty(context, module_obj,
+                               "__file__",
+                               STRING_TO_JSVAL(JS_NewStringCopyZ(context, full_path)),
+                               NULL, NULL,
+                               /* don't set ENUMERATE since we wouldn't want to copy
+                                * this symbol to any other object for example.
+                                */
+                               JSPROP_READONLY | JSPROP_PERMANENT))
+            return JS_FALSE;
+    }
+
     if (!JS_DefineProperty(context, module_obj,
                            "__moduleName__",
                            parent_is_module ?
@@ -228,7 +241,7 @@ import_native_file(JSContext  *context,
     gjs_debug(GJS_DEBUG_IMPORTER,
               "Importing '%s' from '%s'", name, full_path ? full_path : "<internal>");
 
-    module_obj = JS_ConstructObject(context, NULL, NULL, NULL);
+    module_obj = JS_NewObject(context, NULL, NULL, NULL);
     if (module_obj == NULL) {
         return JS_FALSE;
     }
@@ -241,7 +254,7 @@ import_native_file(JSContext  *context,
     if (!define_import(context, obj, module_obj, name))
         return JS_FALSE;
 
-    if (!define_meta_properties(context, module_obj, name, obj))
+    if (!define_meta_properties(context, module_obj, full_path, name, obj))
         goto out;
 
     if (!gjs_import_native_module(context, module_obj, full_path, &flags))
@@ -267,6 +280,7 @@ file_get_utf16_contents (const char  *filename,
                          glong       *length,
                          GError     **error)
 {
+    jschar *result;
     gchar *contents;
     gsize utf8_length;
 
@@ -277,8 +291,11 @@ file_get_utf16_contents (const char  *filename,
     /* No cast here, so we get a warning in the
        (impossible?) case that gunichar2 is different
        from jschar */
-    return g_utf8_to_utf16 (contents, utf8_length,
-                            NULL, length, error);
+    result = g_utf8_to_utf16 (contents, utf8_length,
+                              NULL, length, error);
+
+    g_free (contents);
+    return result;
 }
 static JSObject *
 load_module_init(JSContext  *context,
@@ -328,6 +345,8 @@ load_module_init(JSContext  *context,
             !g_error_matches(error, G_FILE_ERROR, G_FILE_ERROR_NOTDIR) &&
             !g_error_matches(error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
             gjs_throw_g_error(context, error);
+        else
+            g_error_free(error);
 
         return NULL;
     }
@@ -422,7 +441,7 @@ import_file(JSContext  *context,
     gjs_debug(GJS_DEBUG_IMPORTER,
               "Importing '%s'", full_path);
 
-    module_obj = JS_ConstructObject(context, NULL, NULL, NULL);
+    module_obj = JS_NewObject(context, NULL, NULL, NULL);
     if (module_obj == NULL) {
         return JS_FALSE;
     }
@@ -430,7 +449,7 @@ import_file(JSContext  *context,
     if (!define_import(context, obj, module_obj, name))
         return JS_FALSE;
 
-    if (!define_meta_properties(context, module_obj, name, obj))
+    if (!define_meta_properties(context, module_obj, full_path, name, obj))
         goto out;
 
     script_len = 0;
@@ -441,6 +460,8 @@ import_file(JSContext  *context,
             !g_error_matches(error, G_FILE_ERROR, G_FILE_ERROR_NOTDIR) &&
             !g_error_matches(error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
             gjs_throw_g_error(context, error);
+        else
+            g_error_free(error);
 
         goto out;
     }
@@ -981,35 +1002,7 @@ importer_new_resolve(JSContext *context,
     return ret;
 }
 
-/* If we set JSCLASS_CONSTRUCT_PROTOTYPE flag, then this is called on
- * the prototype in addition to on each instance. When called on the
- * prototype, "obj" is the prototype, and "retval" is the prototype
- * also, but can be replaced with another object to use instead as the
- * prototype. If we don't set JSCLASS_CONSTRUCT_PROTOTYPE we can
- * identify the prototype as an object of our class with NULL private
- * data.
- */
-GJS_NATIVE_CONSTRUCTOR_DECLARE(importer)
-{
-    GJS_NATIVE_CONSTRUCTOR_VARIABLES(importer)
-    Importer *priv;
-
-    GJS_NATIVE_CONSTRUCTOR_PRELUDE(importer);
-
-    priv = g_slice_new0(Importer);
-
-    GJS_INC_COUNTER(importer);
-
-    g_assert(priv_from_js(context, object) == NULL);
-    JS_SetPrivate(context, object, priv);
-
-    gjs_debug_lifecycle(GJS_DEBUG_IMPORTER,
-                        "importer constructor, obj %p priv %p", object, priv);
-
-    GJS_NATIVE_CONSTRUCTOR_FINISH(importer);
-
-    return JS_TRUE;
-}
+GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT(importer)
 
 static void
 importer_finalize(JSContext *context,
@@ -1021,7 +1014,7 @@ importer_finalize(JSContext *context,
     gjs_debug_lifecycle(GJS_DEBUG_IMPORTER,
                         "finalize, obj %p priv %p", obj, priv);
     if (priv == NULL)
-        return; /* we are the prototype, not a real instance, so constructor never called */
+        return; /* we are the prototype, not a real instance */
 
     GJS_DEC_COUNTER(importer);
     g_slice_free(Importer, priv);
@@ -1030,11 +1023,6 @@ importer_finalize(JSContext *context,
 /* The bizarre thing about this vtable is that it applies to both
  * instances of the object, and to the prototype that instances of the
  * class have.
- *
- * Also, there's a constructor field in here, but as far as I can
- * tell, it would only be used if no constructor were provided to
- * JS_InitClass. The constructor from JS_InitClass is not applied to
- * the prototype unless JSCLASS_CONSTRUCT_PROTOTYPE is in flags.
  */
 static struct JSClass gjs_importer_class = {
     "GjsFileImporter",
@@ -1104,9 +1092,19 @@ importer_new(JSContext    *context)
                   gjs_importer_class.name, prototype);
     }
 
-    importer = JS_ConstructObject(context, &gjs_importer_class, NULL, global);
+    importer = JS_NewObject(context, &gjs_importer_class, NULL, global);
     if (importer == NULL)
-        gjs_fatal("No memory to create ns object");
+        gjs_fatal("No memory to create importer importer");
+
+    priv = g_slice_new0(Importer);
+
+    GJS_INC_COUNTER(importer);
+
+    g_assert(priv_from_js(context, importer) == NULL);
+    JS_SetPrivate(context, importer, priv);
+
+    gjs_debug_lifecycle(GJS_DEBUG_IMPORTER,
+                        "importer constructor, obj %p priv %p", importer, priv);
 
     return importer;
 }
@@ -1195,7 +1193,7 @@ gjs_define_importer(JSContext    *context,
 
     g_strfreev(search_path);
 
-    if (!define_meta_properties(context, importer, importer_name, in_object))
+    if (!define_meta_properties(context, importer, NULL, importer_name, in_object))
         gjs_fatal("failed to define meta properties on importer");
 
     if (!JS_DefineProperty(context, in_object,
