@@ -43,7 +43,6 @@
 #include <config.h>
 #include <glib.h>
 #include <string.h>
-#include <jsdbgapi.h>
 #include "context.h"
 #include "compat.h"
 #include "jsapi-util.h"
@@ -253,8 +252,6 @@ gjstest_test_func_gjs_stack_dump(void)
 {
   GjsContext *context;
 
-  g_type_init();
-
   /* TODO this test could be better - maybe expose dumpstack as a JS API
    * so that we have a JS stack to dump?  At least here we're getting some
    * coverage.
