@@ -645,9 +645,7 @@ invoke_js_async_from_dbus(JSContext   *context,
                       "conversion of dbus method arg failed but no exception was set?");
     }
 
-    if (argv) {
-        gjs_unroot_value_locations(context, argv, argc);
-    }
+    gjs_rooted_array_free(context, values, TRUE);
 
     return reply;
 }
@@ -1697,38 +1695,7 @@ exports_new_resolve(JSContext *context,
     return JS_TRUE;
 }
 
-/* If we set JSCLASS_CONSTRUCT_PROTOTYPE flag, then this is called on
- * the prototype in addition to on each instance. When called on the
- * prototype, "obj" is the prototype, and "retval" is the prototype
- * also, but can be replaced with another object to use instead as the
- * prototype. If we don't set JSCLASS_CONSTRUCT_PROTOTYPE we can
- * identify the prototype as an object of our class with NULL private
- * data.
- */
-GJS_NATIVE_CONSTRUCTOR_DECLARE(js_exports)
-{
-    GJS_NATIVE_CONSTRUCTOR_VARIABLES(js_exports)
-    Exports *priv;
-
-    GJS_NATIVE_CONSTRUCTOR_PRELUDE(js_exports);
-
-    priv = g_slice_new0(Exports);
-
-    GJS_INC_COUNTER(dbus_exports);
-
-    g_assert(priv_from_js(context, object) == NULL);
-    JS_SetPrivate(context, object, priv);
-
-    gjs_debug_lifecycle(GJS_DEBUG_DBUS,
-                        "exports constructor, obj %p priv %p", object, priv);
-
-    priv->runtime = JS_GetRuntime(context);
-    priv->object = object;
-
-    GJS_NATIVE_CONSTRUCTOR_FINISH(js_exports);
-
-    return JS_TRUE;
-}
+GJS_NATIVE_CONSTRUCTOR_DEFINE_ABSTRACT(js_exports)
 
 static JSBool
 add_connect_funcs(JSContext  *context,
@@ -1788,11 +1755,6 @@ exports_finalize(JSContext *context,
 /* The bizarre thing about this vtable is that it applies to both
  * instances of the object, and to the prototype that instances of the
  * class have.
- *
- * Also, there's a constructor field in here, but as far as I can
- * tell, it would only be used if no constructor were provided to
- * JS_InitClass. The constructor from JS_InitClass is not applied to
- * the prototype unless JSCLASS_CONSTRUCT_PROTOTYPE is in flags.
  */
 static struct JSClass gjs_js_exports_class = {
     "DBusExports", /* means "new DBusExports()" works */
@@ -1825,6 +1787,7 @@ static JSObject*
 exports_new(JSContext  *context,
             DBusBusType which_bus)
 {
+    Exports *priv;
     JSObject *exports;
     JSObject *global;
 
@@ -1865,8 +1828,20 @@ exports_new(JSContext  *context,
                   gjs_js_exports_class.name, prototype);
     }
 
-    exports = JS_ConstructObject(context, &gjs_js_exports_class, NULL, global);
-    /* may be NULL */
+    exports = JS_NewObject(context, &gjs_js_exports_class, NULL, global);
+
+    priv = g_slice_new0(Exports);
+
+    GJS_INC_COUNTER(dbus_exports);
+
+    g_assert(priv_from_js(context, exports) == NULL);
+    JS_SetPrivate(context, exports, priv);
+
+    gjs_debug_lifecycle(GJS_DEBUG_DBUS,
+                        "exports constructor, obj %p priv %p", exports, priv);
+
+    priv->runtime = JS_GetRuntime(context);
+    priv->object = exports;
 
     return exports;
 }
