@@ -1644,7 +1644,7 @@ gjs_js_dbus_get_current_message_context(JSContext  *context,
 
     current_message = _gjs_current_dbus_messages->data;
 
-    context_obj = JS_ConstructObject(context, NULL, NULL, NULL);
+    context_obj = JS_NewObject(context, NULL, NULL, NULL);
     if (context_obj == NULL)
         return JS_FALSE;
 
@@ -1693,7 +1693,7 @@ define_bus_proto(JSContext *context,
     bus_proto_val = JSVAL_VOID;
     JS_AddValueRoot(context, &bus_proto_val);
 
-    bus_proto_obj = JS_ConstructObject(context, NULL, NULL, NULL);
+    bus_proto_obj = JS_NewObject(context, NULL, NULL, NULL);
     if (bus_proto_obj == NULL)
         goto out;
 
@@ -1812,16 +1812,9 @@ define_bus_object(JSContext  *context,
     bus_val = JSVAL_VOID;
     JS_AddValueRoot(context, &bus_val);
 
-    bus_obj = JS_ConstructObject(context, NULL, NULL, NULL);
+    bus_obj = JS_NewObject(context, NULL, proto_obj, NULL);
     if (bus_obj == NULL)
         goto out;
-    /* We need to use a separate call to SetPrototype to work
-     * around a SpiderMonkey bug where with clasp=NULL, the
-     * parent and proto arguments to JS_ConstructObject are
-     * lost.
-     * https://bugzilla.mozilla.org/show_bug.cgi?id=599651
-     */
-    JS_SetPrototype(context, bus_obj, proto_obj);
 
     bus_val = OBJECT_TO_JSVAL(bus_obj);
 
